clear all
clc
close all

load data.txt


global A B n M_amin M_H2O rho_amin rho_water n_p

%% ----Systemvilkaar----

M_amin=119.16;                               
% Molar masse til MDEA [g/mol]

M_H2O=18.02;
% Molar masse til vatn [g/mol]

rho_amin=1038;                              
% Tettleik til MDEA [g/L]

rho_water=917;      
% Tettleik til vatn [g/L]

% Det er g?tt ut i fraa at temperaturavhengigheita til tettleiken 
% er neglisjerbar


%% ----Maaleverdiar----

lnp = log(data(:,3));                  
% CO2-partialtrykk CO2 [kPa] 

wt = data(:,1);                         
% Vektfraksjon MEA [wt%] 

[m c_H2O] = wtToConc(wt);                
% Konsentrasjon MEA [mol/L] og konsentrasjon av vatn [mol/L] 

y = data(:,2);                            
% Ladningsgrad [mol/mol MEA]

T = data(:,4)+273;  
% Temperatur [K]

n = length(y);
% lengden av datapunkta                                 
    

%% ----Reaksjonsbetingelsar----

% Reaksjonsmatrise; Reaksjon mellom ikkje-flyktige komponentar i
% veaskefasen

B= [  -1         %amine          b1  
       1         %amineH +       b2                        
      -1         %HCO3 -         b4  
       1 ];      %CO3 2-         b5                       
     


% Reaksjonsmatride; Reaksjon mellom ikkje-flyktige komponentar og
% flyktig komponent i veaskefasen

A= [ -1          %amine          b1  
      1          %amineH +       b2                
      1          %HCO3 -         b4  
      0];        %CO3 2-         b5             


%% ----Initialkonsentrasjon

%      b1       b2    b3       b4       
b0 = [(1-y).*m  y.*m  y.*m     zeros(n,1)] ;
b0 = b0+1e-5;

  

  
%% ----Estimering av likevektsparameterar----

X=[m y T c_H2O b0];
% Matrise med dei forklarande variablane

a0=[276.0735      -351.514  ]';  
b0=[-10457.3        8143.3  ]';
c0=[-43.0148        51.8474 ]';
% Litteratur, [Hessen 2010]

n_p=length(a0);
% Antal parameterar 

beta0 = [a0;b0;c0];
% Initialvektor

options=optimset('display','iter-detailed','FunValCheck',...
    'on','TolX',1e-10,'TolFun',1e-07,'TolCon',1e-07);

[beta,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = fmincon(@(beta) ...
    fun(beta,X,lnp),beta0,[],[],[],[]...
    ,[],[],@(beta) nonlcon(beta,X),options);

a=beta(1:2);
b=beta(3:4);
c=beta(5:6);
% Estimerte likevektsparameterar

lnp_hat=model(beta,X);                       
% Logaritmisk CO2-partialtrykk, kalkulert ved hjelp av 
% likevktsmodellen    


%% ----Konfidenseinterval----
H=HESSIAN(1:6,1:6);
inv_H=inv(H);
diag_H=diag(inv_H);

e=lnp-lnp_hat;

SSE=e'*e;

TSS=(lnp-mean(lnp))'*(lnp-mean(lnp));

sigma=sqrt(SSE/(n-9));

t=tinv(0.975,(n-9));

conf=t*sqrt(2*diag_H)*sigma;

%% ----Determinasjonskoeffisient----

R2=1-(SSE/TSS);
R2_adj=1-(SSE/(n-9))/(TSS/(n-1));


%% ----AARD----
% average absolute relative deviation

AARD=100*(1/n)*sum(abs(exp(lnp_hat)-exp(lnp))./exp(lnp));



