clear all
clc
close all

load data.txt


global A B n M_amin M_H2O rho_amin rho_water n_p


%% ----Systemvilkaar----

M_amin=61.08;                               
% Molar masse til MEA [g/mol]

M_H2O=18.02;
% Molar masse til vatn [g/mol]

rho_amin=1012;                              
% Tettleik til MEA [g/L]

rho_water=917;      
% Tettleik til vatn [g/L]

% Det er g?tt ut i fraa at temperaturavhengigheita til tettleiken 
% er neglisjerbar


%% ----Maaleverdiar----

lnp = log(data(:,1));                        
% Logaritmisk CO2-partialtrykk [kPa]       

wt = data(:,3);                              
% Vektfraksjon MEA  [wt%]

m = wtToConc(wt);                       
% Konsentrasjon MEA [mol/L]

y = data(:,2);                               
% Ladningsgrad [mol/nmol]  

T = data(:,4)+273;                           
% Temperatur [K]    

n=length(y);                                 
% Lengden av datapunkta                                 
    

%% ----Reaksjonsmatriser----

% Reaksjonsmatrise; Reaksjon mellom ikkje-flyktige komponentar i
% veaskefasen

B= [  -1    -1      % MEA          b1  
       1     0      % MEAH +       b2 
       0     1      % MEACOO -     b3                        
      -1    -1      % HCO3 -       b4  
       1     0];    % CO3 2-       b5    

% Reaksjonsmatrise; Reaksjon mellom ikkje-flyktige komponentar og 
% flyktig komponent i veaskefasen

A= [ -2             % MEA          b1  
      1             % MEAH +       b2 
      1             % MEACOO -     b3                     
      0             % HCO3 -       b4  
      0];           % CO3 2-       b5             

%% ----Initialkonsentrasjon----

%      b1       b2    b3          b4    b5           
b0 = [(1-y).*m  y.*m  zeros(n,1)  y.*m  zeros(n,1)]; 
%b0=b0+1e-5;
  

%% ----Estimering av likevektsparameterane----

X=[m y T b0];                                   
% Matrise med dei forklarande variablane

a0=[220.95       -2.89       -69.68  ]';  
b0=[-6265        3635.58     -6186.21]';   
c0=[-35.48       0           14.82   ]';
%Litteratur, [Hessen 2010] 

n_p=length(a0);
% Antal parameterar 

load xi.txt
%xi=InitialXi([a0;b0;c0],X);
% Initialverdi til reaksjonsomfanget 

beta0 = [a0;b0;c0;xi(:,1);xi(:,2)];          
% Initilvektor

K_LB=-ones(9,1)*inf; 
xi1_LB=ones(n,1)*1e-8;
xi2_LB=ones(n,1)*0.1;
LB=[K_LB;xi1_LB;xi2_LB];
% Nedre granse til parameterane

K_UB=ones(9,1)*inf; 
xi1_UB=ones(n,1)*1e-4;
xi2_UB=ones(n,1)*3;
UB=[K_UB;xi1_UB;xi2_UB];
% oevre grense til parameterane


options=optimset('display','iter-detailed','FunValCheck',...
    'on','TolX',1e-10,'TolFun',1e-09,'TolCon',1e-08);


[beta,FVAL,EXITFLAG,OUTPUT,LAMBDA,GRAD,HESSIAN] = ...
    fmincon(@(beta) fun(beta,X,lnp),beta0,[],[],[],[]...
    ,LB,UB,@(beta) nonlcon(beta,X),options);

a=beta(1:3);                               
b=beta(4:6);                           
c=beta(7:9);
% Estimerte likevektsparameterat


lnp_hat=model(beta,X);                       
% Logaritmisk CO2-partialtrykk, kalkulert ved hjelp av 
% likevktsmodellen    


%% ----Konfidenseinterval----
H=HESSIAN(1:9,1:9);
inv_H=inv(H);
diag_H=diag(inv_H);

e=lnp-lnp_hat;

SSE=e'*e;

TSS=(lnp-mean(lnp))'*(lnp-mean(lnp));

sigma=sqrt(SSE/(n-9));

t=tinv(0.975,(n-9));

conf=t*sqrt(2*diag_H)*sigma;

%% ----Determinasjonskoeffisient----

R2=1-(SSE/TSS);
R2_adj=1-(SSE/(n-9))/(TSS/(n-1));


%% ----AARD----
% average absolute relative deviation

AARD=100*(1/n)*sum(abs(exp(lnp_hat)-exp(lnp))./exp(lnp));


